@@ -10,13 +10,10 @@
  * daniel@veillard.com
  */
 
+#define IN_LIBXSLT
 #include "libxslt.h"
 
 #include <string.h>
-#ifdef HAVE_TIME_H
-#define __USE_XOPEN
-#include <time.h>
-#endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -40,9 +37,9 @@
 #endif
 
 /************************************************************************
- * 									*
- * 		Handling of XSLT debugging				*
- * 									*
+ *									*
+ *		Handling of XSLT debugging				*
+ *									*
  ************************************************************************/
 
 /**
@@ -61,45 +58,53 @@ xsltDebug(xsltTransformContextPtr ctxt, xmlNodePtr node ATTRIBUTE_UNUSED,
 {
     int i, j;
 
-    fprintf(stdout, "Templates:\n");
+    xsltGenericError(xsltGenericErrorContext, "Templates:\n");
     for (i = 0, j = ctxt->templNr - 1; ((i < 15) && (j >= 0)); i++, j--) {
-        fprintf(stdout, "#%d ", i);
+        xsltGenericError(xsltGenericErrorContext, "#%d ", i);
         if (ctxt->templTab[j]->name != NULL)
-            fprintf(stdout, "name %s ", ctxt->templTab[j]->name);
+            xsltGenericError(xsltGenericErrorContext, "name %s ",
+                             ctxt->templTab[j]->name);
         if (ctxt->templTab[j]->match != NULL)
-            fprintf(stdout, "name %s ", ctxt->templTab[j]->match);
+            xsltGenericError(xsltGenericErrorContext, "name %s ",
+                             ctxt->templTab[j]->match);
         if (ctxt->templTab[j]->mode != NULL)
-            fprintf(stdout, "name %s ", ctxt->templTab[j]->mode);
-        fprintf(stdout, "\n");
+            xsltGenericError(xsltGenericErrorContext, "name %s ",
+                             ctxt->templTab[j]->mode);
+        xsltGenericError(xsltGenericErrorContext, "\n");
     }
-    fprintf(stdout, "Variables:\n");
+    xsltGenericError(xsltGenericErrorContext, "Variables:\n");
     for (i = 0, j = ctxt->varsNr - 1; ((i < 15) && (j >= 0)); i++, j--) {
         xsltStackElemPtr cur;
 
         if (ctxt->varsTab[j] == NULL)
             continue;
-        fprintf(stdout, "#%d\n", i);
+        xsltGenericError(xsltGenericErrorContext, "#%d\n", i);
         cur = ctxt->varsTab[j];
         while (cur != NULL) {
             if (cur->comp == NULL) {
-                fprintf(stdout, "corrupted !!!\n");
+                xsltGenericError(xsltGenericErrorContext,
+                                 "corrupted !!!\n");
             } else if (cur->comp->type == XSLT_FUNC_PARAM) {
-                fprintf(stdout, "param ");
+                xsltGenericError(xsltGenericErrorContext, "param ");
             } else if (cur->comp->type == XSLT_FUNC_VARIABLE) {
-                fprintf(stdout, "var ");
+                xsltGenericError(xsltGenericErrorContext, "var ");
             }
             if (cur->name != NULL)
-                fprintf(stdout, "%s ", cur->name);
+                xsltGenericError(xsltGenericErrorContext, "%s ",
+                                 cur->name);
             else
-                fprintf(stdout, "noname !!!!");
+                xsltGenericError(xsltGenericErrorContext, "noname !!!!");
 #ifdef LIBXML_DEBUG_ENABLED
             if (cur->value != NULL) {
-                xmlXPathDebugDumpObject(stdout, cur->value, 1);
+                if ((xsltGenericDebugContext == stdout) ||
+                    (xsltGenericDebugContext == stderr))
+                    xmlXPathDebugDumpObject((FILE*)xsltGenericDebugContext,
+                                            cur->value, 1);
             } else {
-                fprintf(stdout, "NULL !!!!");
+                xsltGenericError(xsltGenericErrorContext, "NULL !!!!");
             }
 #endif
-            fprintf(stdout, "\n");
+            xsltGenericError(xsltGenericErrorContext, "\n");
             cur = cur->next;
         }
 
@@ -107,9 +112,9 @@ xsltDebug(xsltTransformContextPtr ctxt, xmlNodePtr node ATTRIBUTE_UNUSED,
 }
 
 /************************************************************************
- * 									*
- * 		Classic extensions as described by M. Kay		*
- * 									*
+ *									*
+ *		Classic extensions as described by M. Kay		*
+ *									*
  ************************************************************************/
 
 /**
@@ -125,8 +130,7 @@ xsltDebug(xsltTransformContextPtr ctxt, xmlNodePtr node ATTRIBUTE_UNUSED,
 void
 xsltFunctionNodeSet(xmlXPathParserContextPtr ctxt, int nargs){
     if (nargs != 1) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-        xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 		"node-set() : expects one result-tree arg\n");
 	ctxt->error = XPATH_INVALID_ARITY;
 	return;
@@ -134,8 +138,7 @@ xsltFunctionNodeSet(xmlXPathParserContextPtr ctxt, int nargs){
     if ((ctxt->value == NULL) ||
 	((ctxt->value->type != XPATH_XSLT_TREE) &&
 	 (ctxt->value->type != XPATH_NODESET))) {
-	xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-	xsltGenericError(xsltGenericErrorContext,
+	xsltTransformError(xsltXPathGetTransformContext(ctxt), NULL, NULL,
 	    "node-set() invalid arg expecting a result tree\n");
 	ctxt->error = XPATH_INVALID_TYPE;
 	return;
@@ -145,132 +148,6 @@ xsltFunctionNodeSet(xmlXPathParserContextPtr ctxt, int nargs){
     }
 }
 
-
-/*
- * Okay the following really seems unportable and since it's not
- * part of any standard I'm not too ashamed to do this
- */
-#if defined(linux) || defined(__sun)
-#if defined(HAVE_MKTIME) && defined(HAVE_LOCALTIME) && defined(HAVE_ASCTIME)
-#define WITH_LOCALTIME
-
-/**
- * xsltFunctionLocalTime:
- * @ctxt:  the XPath Parser context
- * @nargs:  the number of arguments
- *
- * Implement the localTime XSLT function used by NORM
- *   string localTime(???)
- *
- * This function is available in Norm's extension namespace
- * Code (and comments) contributed by Norm
- */
-static void
-xsltFunctionLocalTime(xmlXPathParserContextPtr ctxt, int nargs) {
-    xmlXPathObjectPtr obj;
-    char *str;
-    char digits[5];
-    char result[29];
-    long int field;
-    time_t gmt, lmt;
-    struct tm gmt_tm;
-    struct tm *local_tm;
- 
-    if (nargs != 1) {
-       xsltPrintErrorContext(xsltXPathGetTransformContext(ctxt), NULL, NULL);
-       xsltGenericError(xsltGenericErrorContext,
-                      "localTime() : invalid number of args %d\n", nargs);
-       ctxt->error = XPATH_INVALID_ARITY;
-       return;
-    }
- 
-    obj = valuePop(ctxt);
-
-    if (obj->type != XPATH_STRING) {
-	obj = xmlXPathConvertString(obj);
-    }
-    if (obj == NULL) {
-	valuePush(ctxt, xmlXPathNewString((const xmlChar *)""));
-	return;
-    }
-    
-    str = (char *) obj->stringval;
-
-    /* str = "$Date: 2001/10/09 17:51:47 $" */
-    memset(digits, 0, sizeof(digits));
-    strncpy(digits, str+7, 4);
-    field = strtol(digits, NULL, 10);
-    gmt_tm.tm_year = field - 1900;
-
-    memset(digits, 0, sizeof(digits));
-    strncpy(digits, str+12, 2);
-    field = strtol(digits, NULL, 10);
-    gmt_tm.tm_mon = field - 1;
-
-    memset(digits, 0, sizeof(digits));
-    strncpy(digits, str+15, 2);
-    field = strtol(digits, NULL, 10);
-    gmt_tm.tm_mday = field;
-
-    memset(digits, 0, sizeof(digits));
-    strncpy(digits, str+18, 2);
-    field = strtol(digits, NULL, 10);
-    gmt_tm.tm_hour = field;
-
-    memset(digits, 0, sizeof(digits));
-    strncpy(digits, str+21, 2);
-    field = strtol(digits, NULL, 10);
-    gmt_tm.tm_min = field;
-
-    memset(digits, 0, sizeof(digits));
-    strncpy(digits, str+24, 2);
-    field = strtol(digits, NULL, 10);
-    gmt_tm.tm_sec = field;
-
-    /* Now turn gmt_tm into a time. */
-    gmt = mktime(&gmt_tm);
-
-
-    /*
-     * FIXME: it's been too long since I did manual memory management.
-     * (I swore never to do it again.) Does this introduce a memory leak?
-     */
-    local_tm = localtime(&gmt);
-
-    /*
-     * Calling localtime() has the side-effect of setting timezone.
-     * After we know the timezone, we can adjust for it
-     */
-    lmt = gmt - timezone;
-
-    /*
-     * FIXME: it's been too long since I did manual memory management.
-     * (I swore never to do it again.) Does this introduce a memory leak?
-     */
-    local_tm = localtime(&lmt);
-
-    /*
-     * Now convert local_tm back into a string. This doesn't introduce
-     * a memory leak, so says asctime(3).
-     */
-
-    str = asctime(local_tm);           /* "Tue Jun 26 05:02:16 2001" */
-                                       /*  0123456789 123456789 123 */
-
-    memset(result, 0, sizeof(result)); /* "Thu, 26 Jun 2001" */
-                                       /*  0123456789 12345 */
-
-    strncpy(result, str, 20);
-    strcpy(result+20, "???");          /* tzname doesn't work, fake it */
-    strncpy(result+23, str+19, 5);
-
-    /* Ok, now result contains the string I want to send back. */
-    valuePush(ctxt, xmlXPathNewString((xmlChar *)result));
-}
-#endif
-#endif /* linux or sun */
-
-
 /**
  * xsltRegisterExtras:
  * @ctxt:  a XSLT process context
@@ -299,11 +176,6 @@ xsltRegisterAllExtras (void) {
     xsltRegisterExtModuleFunction((const xmlChar *) "node-set",
 				  XSLT_XT_NAMESPACE,
 				  xsltFunctionNodeSet);
-#ifdef WITH_LOCALTIME
-    xsltRegisterExtModuleFunction((const xmlChar *) "localTime",
-				  XSLT_NORM_SAXON_NAMESPACE,
-				  xsltFunctionLocalTime);
-#endif
     xsltRegisterExtModuleElement((const xmlChar *) "debug",
 				 XSLT_LIBXSLT_NAMESPACE,
 				 NULL,
